@@ -4,6 +4,8 @@ const GLib = imports.gi.GLib;
 const Gio = imports.gi.Gio;
 const St = imports.gi.St;
 
+const Config = imports.misc.config;
+
 const ExtensionState = {
     ENABLED: 1,
     DISABLED: 2,
@@ -25,6 +27,36 @@ var disabledExtensions;
 // GFile for user extensions
 var userExtensionsDir = null;
 
+/**
+ * versionCheck:
+ * @required: an array of versions we're compatible with
+ * @current: the version we have
+ *
+ * Check if a component is compatible for an extension.
+ * @required is an array, and at least one version must match.
+ * @current must be in the format <major>.<minor>.<point>.<micro>
+ * <micro> is always ignored
+ * <point> is ignored if <minor> is even (so you can target the
+ * whole stable release)
+ * <minor> and <major> must match
+ * Each target version must be at least <major> and <minor>
+ */
+function versionCheck(required, current) {
+    let currentArray = current.split('.');
+    let major = currentArray[0];
+    let minor = currentArray[1];
+    let point = currentArray[2];
+    for (let i = 0; i < required.length; i++) {
+        let requiredArray = required[i].split('.');
+        if (requiredArray[0] == major &&
+            requiredArray[1] == minor &&
+            (requiredArray[2] == point ||
+             (requiredArray[2] == undefined && parseInt(minor) % 2 == 0)))
+            return true;
+    }
+    return false;
+}
+
 function loadExtension(dir, enabled, type) {
     let info;
     let baseErrorString = 'While loading extension from "' + dir.get_parse_name() + '": ';
@@ -35,7 +67,13 @@ function loadExtension(dir, enabled, type) {
         return;
     }
 
-    let [success, metadataContents, len, etag] = metadataFile.load_contents(null);
+    let metadataContents;
+    try {
+        metadataContents = Shell.get_file_contents_utf8_sync(metadataFile.get_path());
+    } catch (e) {
+        global.logError(baseErrorString + 'Failed to load metadata.json: ' + e);
+        return;
+    }
     let meta;
     try {
         meta = JSON.parse(metadataContents);
@@ -43,8 +81,8 @@ function loadExtension(dir, enabled, type) {
         global.logError(baseErrorString + 'Failed to parse metadata.json: ' + e);
         return;
     }
-    let requiredProperties = ['uuid', 'name', 'description'];
-    for (let i = 0; i < requiredProperties; i++) {
+    let requiredProperties = ['uuid', 'name', 'description', 'shell-version'];
+    for (let i = 0; i < requiredProperties.length; i++) {
         let prop = requiredProperties[i];
         if (!meta[prop]) {
             global.logError(baseErrorString + 'missing "' + prop + '" property in metadata.json');
@@ -68,6 +106,12 @@ function loadExtension(dir, enabled, type) {
         return;
     }
 
+    if (!versionCheck(meta['shell-version'], Config.PACKAGE_VERSION) ||
+        (meta['js-version'] && !versionCheck(meta['js-version'], Config.GJS_VERSION))) {
+        global.logError(baseErrorString + 'extension is not compatible with current GNOME Shell and/or GJS version');
+        return;
+    }
+
     extensionMeta[meta.uuid] = meta;
     extensionMeta[meta.uuid].type = type;
     extensionMeta[meta.uuid].path = dir.get_path();
@@ -112,7 +156,7 @@ function loadExtension(dir, enabled, type) {
         return;
     }
     try {
-        extensionModule.main();
+        extensionModule.main(meta);
     } catch (e) {
         if (stylesheetPath != null)
             theme.unload_stylesheet(stylesheetPath);
@@ -136,8 +180,15 @@ function init() {
 }
 
 function _loadExtensionsIn(dir, type) {
-    let fileEnum = dir.enumerate_children('standard::*', Gio.FileQueryInfoFlags.NONE, null);
+    let fileEnum;
     let file, info;
+    try {
+        fileEnum = dir.enumerate_children('standard::*', Gio.FileQueryInfoFlags.NONE, null);
+    } catch (e) {
+        global.logError('' + e);
+       return;
+    }
+
     while ((info = fileEnum.next_file(null)) != null) {
         let fileType = info.get_file_type();
         if (fileType != Gio.FileType.DIRECTORY)
