@@ -125,6 +125,7 @@ WindowClone.prototype = {
                                               dragActorMaxSize: WINDOW_DND_SIZE,
                                               dragActorOpacity: DRAGGING_WINDOW_OPACITY });
         this._draggable.connect('drag-begin', Lang.bind(this, this._onDragBegin));
+        this._draggable.connect('drag-cancelled', Lang.bind(this, this._onDragCancelled));
         this._draggable.connect('drag-end', Lang.bind(this, this._onDragEnd));
         this.inDrag = false;
 
@@ -220,8 +221,15 @@ WindowClone.prototype = {
 
         let [width, height] = this.actor.get_transformed_size();
 
-        this.actor.x = _clamp(this.actor.x, 0, global.screen_width  - width);
-        this.actor.y = _clamp(this.actor.y, Panel.PANEL_HEIGHT, global.screen_height - height);
+        let monitorIndex = this.metaWindow.get_monitor();
+        let availArea = global.get_monitors()[monitorIndex];
+        if (monitorIndex == global.get_primary_monitor_index()) {
+            availArea.y += Main.panel.actor.height;
+            availArea.height -= Main.panel.actor.height;
+        }
+
+        this.actor.x = _clamp(this.actor.x, availArea.x, availArea.x + availArea.width - width);
+        this.actor.y = _clamp(this.actor.y, availArea.y, availArea.y + availArea.height - height);
     },
 
     _zoomStart : function () {
@@ -288,10 +296,16 @@ WindowClone.prototype = {
     },
 
     _onDragBegin : function (draggable, time) {
+        [this.dragOrigX, this.dragOrigY] = this.actor.get_position();
+        this.dragOrigScale = this.actor.scale_x;
         this.inDrag = true;
         this.emit('drag-begin');
     },
 
+    _onDragCancelled : function (draggable, time) {
+        this.emit('drag-cancelled');
+    },
+
     _onDragEnd : function (draggable, time, snapback) {
         this.inDrag = false;
 
@@ -327,6 +341,7 @@ WindowOverlay.prototype = {
 
         this._windowClone = windowClone;
         this._parentActor = parentActor;
+        this._hidden = false;
 
         let title = new St.Label({ style_class: 'window-caption',
                                    text: metaWindow.title });
@@ -372,23 +387,19 @@ WindowOverlay.prototype = {
     },
 
     hide: function() {
+        this._hidden = true;
         this.closeButton.hide();
         this.title.hide();
     },
 
     show: function() {
-        let [x, y, mask] = global.get_pointer();
-        let actor = global.stage.get_actor_at_pos(Clutter.PickMode.REACTIVE,
-                                                  x, y);
-        if (actor == this._windowClone.actor) {
-            this.closeButton.show();
-        }
+        this._hidden = false;
         this.title.show();
     },
 
     fadeIn: function() {
+        this.show();
         this.title.opacity = 0;
-        this.title.show();
         this._parentActor.raise_top();
         Tweener.addTween(this.title,
                         { opacity: 255,
@@ -480,6 +491,12 @@ WindowOverlay.prototype = {
     },
 
     _onEnter: function() {
+        // We might get enter events on the clone while the overlay is
+        // hidden, e.g. during animations, we ignore these events,
+        // as the close button will be shown as needed when the overlays
+        // are shown again
+        if (this._hidden)
+            return;
         this._parentActor.raise_top();
         this.closeButton.show();
         this.emit('show-close-button');
@@ -520,44 +537,45 @@ WindowOverlay.prototype = {
 Signals.addSignalMethods(WindowOverlay.prototype);
 
 const WindowPositionFlags = {
-    ZOOM: 1 << 0,
+    INITIAL: 1 << 0,
     ANIMATE: 1 << 1
 };
 
 /**
- * @metaWorkspace: a #Meta.Workspace
+ * @metaWorkspace: a #Meta.Workspace, or null
  */
-function Workspace(metaWorkspace) {
-    this._init(metaWorkspace);
+function Workspace(metaWorkspace, monitorIndex) {
+    this._init(metaWorkspace, monitorIndex);
 }
 
 Workspace.prototype = {
-    _init : function(metaWorkspace) {
+    _init : function(metaWorkspace, monitorIndex) {
         // When dragging a window, we use this slot for reserve space.
         this._reservedSlot = null;
         this.metaWorkspace = metaWorkspace;
+        this._x = 0;
+        this._y = 0;
+        this._width = 0;
+        this._height = 0;
+
+        this.monitorIndex = monitorIndex;
+        this._monitor = global.get_monitors()[this.monitorIndex];
         this._windowOverlaysGroup = new Clutter.Group();
         // Without this the drop area will be overlapped.
         this._windowOverlaysGroup.set_size(0, 0);
 
         this.actor = new Clutter.Group();
-        this.actor._delegate = this;
+        this.actor.set_size(0, 0);
+
+        this._dropRect = new Clutter.Rectangle({ opacity: 0 });
+        this._dropRect._delegate = this;
+
+        this.actor.add_actor(this._dropRect);
+        this.actor.add_actor(this._windowOverlaysGroup);
 
         this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
 
-        // Items in _windowOverlaysGroup should not be scaled, so we don't
-        // add them to this.actor, but to its parent whenever it changes
-        this.actor.connect('parent-set', Lang.bind(this, this._onParentSet));
-
-        // Auto-sizing is unreliable in the presence of ClutterClone, so rather than
-        // implicitly counting on the workspace actor to be sized to the size of the
-        // included desktop actor clone, set the size explicitly to the screen size.
-        // See http://bugzilla.openedhand.com/show_bug.cgi?id=1755
-        this.actor.width = global.screen_width;
-        this.actor.height = global.screen_height;
-        this.scale = 1.0;
-
-        let windows = Main.getWindowActorsForWorkspace(this.metaWorkspace.index());
+        let windows = global.get_window_actors().filter(this._isMyWindow, this);
 
         // Create clones for windows that should be
         // visible in the Overview
@@ -570,15 +588,37 @@ Workspace.prototype = {
         }
 
         // Track window changes
-        this._windowAddedId = this.metaWorkspace.connect('window-added',
-                                                          Lang.bind(this, this._windowAdded));
-        this._windowRemovedId = this.metaWorkspace.connect('window-removed',
-                                                            Lang.bind(this, this._windowRemoved));
+        if (this.metaWorkspace) {
+            this._windowAddedId = this.metaWorkspace.connect('window-added',
+                                                             Lang.bind(this, this._windowAdded));
+            this._windowRemovedId = this.metaWorkspace.connect('window-removed',
+                                                               Lang.bind(this, this._windowRemoved));
+        }
+        this._windowEnteredMonitorId = global.screen.connect('window-entered-monitor',
+                                                           Lang.bind(this, this._windowEnteredMonitor));
+        this._windowLeftMonitorId = global.screen.connect('window-left-monitor',
+                                                           Lang.bind(this, this._windowLeftMonitor));
         this._repositionWindowsId = 0;
 
         this.leavingOverview = false;
     },
 
+    setGeometry: function(x, y, width, height) {
+        this._x = x;
+        this._y = y;
+        this._width = width;
+        this._height = height;
+
+        // This is sometimes called during allocation, so we do this later
+        Meta.later_add(Meta.LaterType.BEFORE_REDRAW, Lang.bind(this,
+            function () {
+                this._dropRect.set_position(x, y);
+                this._dropRect.set_size(width, height);
+                return false;
+            }));
+
+    },
+
     _lookupIndex: function (metaWindow) {
         for (let i = 0; i < this._windows.length; i++) {
             if (this._windows[i].metaWindow == metaWindow) {
@@ -588,17 +628,6 @@ Workspace.prototype = {
         return -1;
     },
 
-    _onParentSet: function(actor, old_parent) {
-        let new_parent = this.actor.get_parent();
-        if (new_parent == null)
-            return;
-
-        if (old_parent)
-            this._windowOverlaysGroup.reparent(new_parent);
-        else
-            new_parent.add_actor(this._windowOverlaysGroup);
-    },
-
     containsMetaWindow: function (metaWindow) {
         return this._lookupIndex(metaWindow) >= 0;
     },
@@ -677,10 +706,20 @@ Workspace.prototype = {
         let xDelta, yDelta, distanceSquared;
         let actorWidth, actorHeight;
 
-        actorWidth = actor.width * actor.scale_x;
-        actorHeight = actor.height * actor.scale_y;
-        xDelta = actor.x + actorWidth / 2.0 - xCenter * global.screen_width;
-        yDelta = actor.y + actorHeight / 2.0 - yCenter * global.screen_height;
+        let x = actor.x;
+        let y = actor.y;
+        let scale = actor.scale_x;
+
+        if (actor._delegate.inDrag) {
+            x = actor._delegate.dragOrigX;
+            y = actor._delegate.dragOrigY;
+            scale = actor._delegate.dragOrigScale;
+        }
+
+        actorWidth = actor.width * scale;
+        actorHeight = actor.height * scale;
+        xDelta = x + actorWidth / 2.0 - xCenter * this._width - this._x;
+        yDelta = y + actorHeight / 2.0 - yCenter * this._height - this._y;
         distanceSquared = xDelta * xDelta + yDelta * yDelta;
 
         return distanceSquared;
@@ -705,6 +744,12 @@ Workspace.prototype = {
                 let delta = this._computeWindowMotion(cloneActor, slot);
 
                 motion += delta;
+
+                // Bail out early if we're already larger than the
+                // previous best
+                if (minimumMotionPermutation != null &&
+                    motion > minimumMotion)
+                    continue;
             }
 
             if (minimumMotionPermutation == null || motion < minimumMotion) {
@@ -765,38 +810,35 @@ Workspace.prototype = {
     },
 
     /**
-     * _getSlotRelativeGeometry:
+     * _getSlotGeometry:
      * @slot: A layout slot
      *
-     * Returns: the workspace-relative [x, y, width, height]
+     * Returns: the screen-relative [x, y, width, height]
      * of a given window layout slot.
      */
-    _getSlotRelativeGeometry: function(slot) {
+    _getSlotGeometry: function(slot) {
         let [xCenter, yCenter, fraction] = slot;
 
-        let width = global.screen_width * fraction;
-        let height = global.screen_height * fraction;
+        let width = this._width * fraction;
+        let height = this._height * fraction;
 
-        let x = xCenter * global.screen_width - width / 2;
-        let y = yCenter * global.screen_height - height / 2;
+        let x = this._x + xCenter * this._width - width / 2 ;
+        let y = this._y + yCenter * this._height - height / 2;
 
         return [x, y, width, height];
     },
 
     /**
-     * _computeWindowRelativeLayout:
+     * _computeWindowLayout:
      * @metaWindow: A #MetaWindow
      * @slot: A layout slot
      *
      * Given a window and slot to fit it in, compute its
-     * workspace-relative [x, y, scale] where scale applies
+     * screen-relative [x, y, scale] where scale applies
      * to both X and Y directions.
      */
-    _computeWindowRelativeLayout: function(metaWindow, slot) {
-        let [xCenter, yCenter, fraction] = slot;
-        let [x, y, width, height] = this._getSlotRelativeGeometry(slot);
-
-        xCenter = xCenter * global.screen_width;
+    _computeWindowLayout: function(metaWindow, slot) {
+        let [x, y, width, height] = this._getSlotGeometry(slot);
 
         let rect = metaWindow.get_outer_rect();
         let buttonOuterHeight, captionHeight;
@@ -804,23 +846,19 @@ Workspace.prototype = {
 
         if (this._windowOverlays[0]) {
             [buttonOuterHeight, captionHeight] = this._windowOverlays[0].chromeHeights();
-            buttonOuterWidth = this._windowOverlays[0].chromeWidth() / this.scale;
+            buttonOuterWidth = this._windowOverlays[0].chromeWidth();
         } else
             [buttonOuterHeight, captionHeight] = [0, 0];
-        buttonOuterHeight /= this.scale;
-        captionHeight /= this.scale;
 
-        let desiredWidth = global.screen_width * fraction;
-        let desiredHeight = global.screen_height * fraction;
-        let scale = Math.min((desiredWidth - buttonOuterWidth) / rect.width,
-                             (desiredHeight - buttonOuterHeight - captionHeight) / rect.height,
-                             1.0 / this.scale);
+        let scale = Math.min((width - buttonOuterWidth) / rect.width,
+                             (height - buttonOuterHeight - captionHeight) / rect.height,
+                             1.0);
 
-        x = Math.floor(xCenter - 0.5 * scale * rect.width);
+        x = Math.floor(x + (width - scale * rect.width) / 2);
 
         // We want to center the window in case we have just one
         if (metaWindow.get_workspace().n_windows == 1)
-            y = Math.floor(yCenter * global.screen_height - 0.5 * scale * rect.height);
+            y = Math.floor(y + (height - scale * rect.height) / 2);
         else
             y = Math.floor(y + height - rect.height * scale - captionHeight);
 
@@ -846,7 +884,7 @@ Workspace.prototype = {
     /**
      * positionWindows:
      * @flags:
-     *  ZOOM - workspace is moving at the same time and we need to take that into account.
+     *  INITIAL - this is the initial positioning of the windows.
      *  ANIMATE - Indicates that we need animate changing position.
      */
     positionWindows : function(flags) {
@@ -859,7 +897,7 @@ Workspace.prototype = {
         if (this._reservedSlot)
             clones.push(this._reservedSlot);
 
-        let workspaceZooming = flags & WindowPositionFlags.ZOOM;
+        let initialPositioning = flags & WindowPositionFlags.INITIAL;
         let animate = flags & WindowPositionFlags.ANIMATE;
 
         // Start the animations
@@ -867,7 +905,7 @@ Workspace.prototype = {
         clones = this._orderWindowsByMotionAndStartup(clones, slots);
 
         let currentWorkspace = global.screen.get_active_workspace();
-        let isOnCurrentWorkspace = this.metaWorkspace == currentWorkspace;
+        let isOnCurrentWorkspace = this.metaWorkspace == null || this.metaWorkspace == currentWorkspace;
 
         for (let i = 0; i < clones.length; i++) {
             let slot = slots[i];
@@ -881,7 +919,7 @@ Workspace.prototype = {
             if (clone.inDrag)
                 continue;
 
-            let [x, y, scale] = this._computeWindowRelativeLayout(metaWindow, slot);
+            let [x, y, scale] = this._computeWindowLayout(metaWindow, slot);
 
             if (overlay)
                 overlay.hide();
@@ -890,7 +928,7 @@ Workspace.prototype = {
                     /* Hidden windows should fade in and grow
                      * therefore we need to resize them now so they
                      * can be scaled up later */
-                     if (workspaceZooming) {
+                     if (initialPositioning) {
                          clone.actor.opacity = 0;
                          clone.actor.scale_x = 0;
                          clone.actor.scale_y = 0;
@@ -911,7 +949,6 @@ Workspace.prototype = {
                                    y: y,
                                    scale_x: scale,
                                    scale_y: scale,
-                                   workspace_relative: workspaceZooming ? this : null,
                                    time: Overview.ANIMATION_TIME,
                                    transition: 'easeOutQuad',
                                    onComplete: Lang.bind(this, function() {
@@ -934,7 +971,7 @@ Workspace.prototype = {
             let clone = clones[i];
             let metaWindow = clone.metaWindow;
             if (i == 0) {
-                clone.setStackAbove(null);
+                clone.setStackAbove(this._dropRect);
             } else {
                 let previousClone = clones[i - 1];
                 clone.setStackAbove(previousClone.actor);
@@ -954,10 +991,8 @@ Workspace.prototype = {
         // be after the workspace animation finishes.
         let [cloneX, cloneY] = clone.actor.get_position();
         let [cloneWidth, cloneHeight] = clone.actor.get_size();
-        cloneX = this.x + this.scale * cloneX;
-        cloneY = this.y + this.scale * cloneY;
-        cloneWidth = this.scale * clone.actor.scale_x * cloneWidth;
-        cloneHeight = this.scale * clone.actor.scale_y * cloneHeight;
+        cloneWidth = clone.actor.scale_x * cloneWidth;
+        cloneHeight = clone.actor.scale_y * cloneHeight;
 
         if (overlay) {
             overlay.updatePositions(cloneX, cloneY, cloneWidth, cloneHeight);
@@ -973,7 +1008,8 @@ Workspace.prototype = {
         for (let i = 0; i < this._windows.length; i++) {
             let clone = this._windows[i];
             let overlay = this._windowOverlays[i];
-            this._showWindowOverlay(clone, overlay, this.metaWorkspace == currentWorkspace);
+            this._showWindowOverlay(clone, overlay,
+                                    this.metaWorkspace == null || this.metaWorkspace == currentWorkspace);
         }
     },
 
@@ -982,12 +1018,10 @@ Workspace.prototype = {
             return true;
 
         let [x, y, mask] = global.get_pointer();
-        let wsWidth = this.actor.width * this.scale;
-        let wsHeight = this.actor.height * this.scale;
 
         let pointerHasMoved = (this._cursorX != x && this._cursorY != y);
-        let inWorkspace = (this.x < x && x < this.x + wsWidth &&
-                           this.y < y && y < this.y + wsHeight);
+        let inWorkspace = (this._x < x && x < this._x + this._width &&
+                           this._y < y && y < this._y + this._height);
 
         if (pointerHasMoved && inWorkspace) {
             // store current cursor position
@@ -1012,7 +1046,7 @@ Workspace.prototype = {
         this._windowOverlaysGroup.hide();
     },
 
-    _windowRemoved : function(metaWorkspace, metaWin) {
+    _doRemoveWindow : function(metaWin) {
         let win = metaWin.get_compositor_private();
 
         // find the position of the window in our list
@@ -1021,6 +1055,10 @@ Workspace.prototype = {
         if (index == -1)
             return;
 
+        // Check if window still should be here
+        if (win && this._isMyWindow(win))
+            return;
+
         let clone = this._windows[index];
 
         this._windows.splice(index, 1);
@@ -1064,7 +1102,7 @@ Workspace.prototype = {
             Lang.bind(this, this._delayedWindowRepositioning));
     },
 
-    _windowAdded : function(metaWorkspace, metaWin) {
+    _doAddWindow : function(metaWin) {
         if (this.leavingOverview)
             return;
 
@@ -1076,30 +1114,62 @@ Workspace.prototype = {
             Mainloop.idle_add(Lang.bind(this,
                                         function () {
                                             if (this.actor && metaWin.get_compositor_private())
-                                                this._windowAdded(metaWorkspace, metaWin);
+                                                this._doAddWindow(metaWin);
                                             return false;
                                         }));
             return;
         }
 
-        if (!this._isOverviewWindow(win))
+        // We might have the window in our list already if it was on all workspaces and
+        // now was moved to this workspace
+        if (this._lookupIndex (metaWin) != -1)
+            return;
+
+        if (!this._isMyWindow(win) || !this._isOverviewWindow(win))
             return;
 
         let clone = this._addWindowClone(win);
 
         if (win._overviewHint) {
-            let x = (win._overviewHint.x - this.actor.x) / this.scale;
-            let y = (win._overviewHint.y - this.actor.y) / this.scale;
-            let scale = win._overviewHint.scale / this.scale;
+            let x = win._overviewHint.x - this.actor.x;
+            let y = win._overviewHint.y - this.actor.y;
+            let scale = win._overviewHint.scale;
             delete win._overviewHint;
 
             clone.actor.set_position (x, y);
             clone.actor.set_scale (scale, scale);
+        } else {
+            // Position new windows at the top corner of the workspace rather
+            // than where they were placed for real to avoid the window
+            // being clipped to the workspaceView. Its not really more
+            // natural for the window to suddenly appear in the overview
+            // on some seemingly random location anyway.
+            clone.actor.set_position (this._x, this._y);
         }
 
         this.positionWindows(WindowPositionFlags.ANIMATE);
     },
 
+    _windowAdded : function(metaWorkspace, metaWin) {
+        this._doAddWindow(metaWin);
+    },
+
+    _windowRemoved : function(metaWorkspace, metaWin) {
+        this._doRemoveWindow(metaWin);
+    },
+
+    _windowEnteredMonitor : function(metaScreen, monitorIndex, metaWin) {
+        if (monitorIndex == this.monitorIndex) {
+            this._doAddWindow(metaWin);
+        }
+    },
+
+    _windowLeftMonitor : function(metaScreen, monitorIndex, metaWin) {
+        if (monitorIndex == this.monitorIndex) {
+            this._doRemoveWindow(metaWin);
+        }
+    },
+
     // check for maximized windows on the workspace
     hasMaximizedWindows: function() {
         for (let i = 0; i < this._windows.length; i++) {
@@ -1114,14 +1184,11 @@ Workspace.prototype = {
 
     // Animate the full-screen to Overview transition.
     zoomToOverview : function() {
-        this.actor.set_position(this.x, this.y);
-        this.actor.set_scale(this.scale, this.scale);
-
         // Position and scale the windows.
         if (Main.overview.animationInProgress)
-            this.positionWindows(WindowPositionFlags.ANIMATE | WindowPositionFlags.ZOOM);
+            this.positionWindows(WindowPositionFlags.ANIMATE | WindowPositionFlags.INITIAL);
         else
-            this.positionWindows(WindowPositionFlags.ZOOM);
+            this.positionWindows(WindowPositionFlags.INITIAL);
     },
 
     // Animates the return from Overview mode
@@ -1139,7 +1206,7 @@ Workspace.prototype = {
         this._overviewHiddenId = Main.overview.connect('hidden', Lang.bind(this,
                                                                            this._doneLeavingOverview));
 
-        if (this._metaWorkspace == currentWorkspace)
+        if (this.metaWorkspace != null && this.metaWorkspace != currentWorkspace)
             return;
 
         // Position and scale the windows.
@@ -1154,7 +1221,6 @@ Workspace.prototype = {
                                    y: clone.origY,
                                    scale_x: 1.0,
                                    scale_y: 1.0,
-                                   workspace_relative: this,
                                    time: Overview.ANIMATION_TIME,
                                    opacity: 255,
                                    transition: 'easeOutQuad'
@@ -1165,7 +1231,6 @@ Workspace.prototype = {
                                  { scale_x: 0,
                                    scale_y: 0,
                                    opacity: 0,
-                                   workspace_relative: this,
                                    time: Overview.ANIMATION_TIME,
                                    transition: 'easeOutQuad'
                                  });
@@ -1185,8 +1250,12 @@ Workspace.prototype = {
         }
         Tweener.removeTweens(actor);
 
-        this.metaWorkspace.disconnect(this._windowAddedId);
-        this.metaWorkspace.disconnect(this._windowRemovedId);
+        if (this.metaWorkspace) {
+            this.metaWorkspace.disconnect(this._windowAddedId);
+            this.metaWorkspace.disconnect(this._windowRemovedId);
+        }
+        global.screen.disconnect(this._windowEnteredMonitorId);
+        global.screen.disconnect(this._windowLeftMonitorId);
 
         if (this._repositionWindowsId > 0)
             Mainloop.source_remove(this._repositionWindowsId);
@@ -1205,9 +1274,10 @@ Workspace.prototype = {
         this.leavingOverview = false;
     },
 
-    // Tests if @win belongs to this workspaces
+    // Tests if @win belongs to this workspaces and monitor
     _isMyWindow : function (win) {
-        return Main.isWindowActorDisplayedOnWorkspace(win, this.metaWorkspace.index());
+        return (this.metaWorkspace == null || Main.isWindowActorDisplayedOnWorkspace(win, this.metaWorkspace.index())) &&
+            (!win.get_meta_window() || win.get_meta_window().get_monitor() == this.monitorIndex);
     },
 
     // Tests if @win should be shown in the Overview
@@ -1228,6 +1298,10 @@ Workspace.prototype = {
                           Main.overview.beginWindowDrag();
                           overlay.hide();
                       }));
+        clone.connect('drag-cancelled',
+                      Lang.bind(this, function(clone) {
+                          Main.overview.cancelledWindowDrag();
+                      }));
         clone.connect('drag-end',
                       Lang.bind(this, function(clone) {
                           Main.overview.endWindowDrag();
@@ -1291,8 +1365,10 @@ Workspace.prototype = {
     },
 
     _onCloneSelected : function (clone, time) {
-        Main.activateWindow(clone.metaWindow, time,
-                            this.metaWorkspace.index());
+        let wsIndex = undefined;
+        if (this.metaWorkspace)
+            wsIndex = this.metaWorkspace.index();
+        Main.activateWindow(clone.metaWindow, time, wsIndex);
     },
 
     // Draggable target interface
@@ -1320,7 +1396,15 @@ Workspace.prototype = {
             };
 
             let metaWindow = win.get_meta_window();
-            metaWindow.change_workspace_by_index(this.metaWorkspace.index(),
+
+            // We need to move the window before changing the workspace, because
+            // the move itself could cause a workspace change if the window enters
+            // the primary monitor
+            if (metaWindow.get_monitor() != this.monitorIndex)
+                metaWindow.move_to_monitor(this.monitorIndex);
+
+            let index = this.metaWorkspace ? this.metaWorkspace.index() : global.screen.get_active_workspace_index();
+            metaWindow.change_workspace_by_index(index,
                                                  false, // don't create workspace
                                                  time);
             return true;
@@ -1335,56 +1419,3 @@ Workspace.prototype = {
 };
 
 Signals.addSignalMethods(Workspace.prototype);
-
-// Create a SpecialPropertyModifier to let us move windows in a
-// straight line on the screen even though their containing workspace
-// is also moving.
-Tweener.registerSpecialPropertyModifier('workspace_relative', _workspaceRelativeModifier, _workspaceRelativeGet);
-
-function _workspaceRelativeModifier(workspace) {
-    let [startX, startY] = Main.overview.getPosition();
-    let overviewPosX, overviewPosY, overviewScale;
-
-    if (!workspace)
-        return [];
-
-    if (workspace.leavingOverview) {
-        let [zoomedInX, zoomedInY] = Main.overview.getZoomedInPosition();
-        overviewPosX = { begin: startX, end: zoomedInX };
-        overviewPosY = { begin: startY, end: zoomedInY };
-        overviewScale = { begin: Main.overview.getScale(),
-                          end: Main.overview.getZoomedInScale() };
-    } else {
-        overviewPosX = { begin: startX, end: 0 };
-        overviewPosY = { begin: startY, end: 0 };
-        overviewScale = { begin: Main.overview.getScale(), end: 1 };
-    }
-
-    return [ { name: 'x',
-               parameters: { workspacePos: workspace.x,
-                             overviewPos: overviewPosX,
-                             overviewScale: overviewScale } },
-             { name: 'y',
-               parameters: { workspacePos: workspace.y,
-                             overviewPos: overviewPosY,
-                             overviewScale: overviewScale } }
-           ];
-}
-
-function _workspaceRelativeGet(begin, end, time, params) {
-    let curOverviewPos = (1 - time) * params.overviewPos.begin +
-                         time * params.overviewPos.end;
-    let curOverviewScale = (1 - time) * params.overviewScale.begin +
-                           time * params.overviewScale.end;
-
-    // Calculate the screen position of the window.
-    let screen = (1 - time) *
-                 ((begin + params.workspacePos) * params.overviewScale.begin +
-                  params.overviewPos.begin) +
-                 time *
-                 ((end + params.workspacePos) * params.overviewScale.end +
-                 params.overviewPos.end);
-
-    // Return the workspace coordinates.
-    return (screen - curOverviewPos) / curOverviewScale - params.workspacePos;
-}
